"""
Timeout decorator.

    :copyright: (c) 2012-2013 by PN.
    :license: MIT, see LICENSE for more details.
"""

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division

import sys
import time
import multiprocessing
import signal
from functools import wraps
import psutil

############################################################
# Timeout
############################################################

# http://www.saltycrane.com/blog/2010/04/
#                                  using-python-timeout-decorator-uploading-s3/
# Used work of Stephen "Zero" Chappell <Noctis.Skytower@gmail.com>
# in https://code.google.com/p/verse-quiz/source/browse/trunk/timeout.py


class MyTimeoutError(AssertionError):

    """Thrown when a timeout occurs in the `timeout` context manager."""

    def __init__(self, value="Timed Out"):  # pylint: disable=W0231
        self.value = value

    def __str__(self):
        return repr(self.value)  # pragma: no cover


def timeout(seconds=None, use_signals=True):
    """Add a timeout parameter to a function and return it.

    :param seconds: optional time limit in seconds or fractions of a second.
        If None is passed, no timeout is applied. This adds some flexibility to
        the usage: you can disable timing out depending on the settings.
    :type seconds: float
    :param use_signals: flag indicating whether signals should be used for
        timing function out or the multiprocessing.
        When using multiprocessing, timeout granularity is limited to 10ths of
        a second.
    :type use_signals: bool

    :raises: MyTimeoutError if time limit is reached

    It is illegal to pass anything other than a function as the first
    parameter. The function is wrapped and returned to the caller.
    """
    def decorate(function):
        """Subfunction"""
        if not seconds:
            return function

        if use_signals:
            def handler(signum, frame):  # pylint: disable=W0613
                """Signal handler"""
                raise MyTimeoutError()

            @wraps(function)
            def new_function(*args, **kwargs):
                """Subfunction"""
                new_seconds = kwargs.pop('timeout', seconds)
                if new_seconds:  # pragma: no cover
                    old = signal.signal(signal.SIGALRM, handler)
                    signal.setitimer(signal.ITIMER_REAL, new_seconds)
                try:
                    return function(*args, **kwargs)
                finally:
                    if new_seconds:  # pragma: no cover
                        signal.setitimer(signal.ITIMER_REAL, 0)
                        signal.signal(signal.SIGALRM, old)
            return new_function
        else:
            return _Timeout(function, seconds)

    return decorate


def _target(queue, function, *args, **kwargs):
    """Run a function with arguments and return output via a queue.

    This is a helper function for the Process created in _Timeout. It runs
    the function with positional arguments and keyword arguments and then
    returns the function's output by way of a queue. If an exception gets
    raised, it is returned to _Timeout to be raised by the value property.
    """
    try:
        queue.put((True, function(*args, **kwargs)))
    except:  # pragma: no cover  # pylint: disable=W0702
        queue.put((False, sys.exc_info()[1]))  # pragma: no cover


def _killchildprocesses(pid):
    """This function will kill all childprocesses of :pid: """

    pproc = psutil.Process(pid)
    for proc in reversed(pproc.children(recursive=True)):
        try:  # pragma: no cover
            proc.terminate()  # pragma: no cover
        except psutil.NoSuchProcess:  # pragma: no cover
            pass  # pragma: no cover
        else:  # pragma: no cover
            try:  # pragma: no cover
                proc.wait(1)  # pragma: no cover
            except psutil.TimeoutExpired:  # pragma: no cover
                try:  # pragma: no cover
                    proc.kill()  # pragma: no cover
                except psutil.NoSuchProcess:  # pragma: no cover
                    pass  # pragma: no cover


class _Timeout:

    """Wrap a function and add a timeout (limit) attribute to it.

    Instances of this class are automatically generated by the add_timeout
    function defined above. Wrapping a function allows asynchronous calls
    to be made and termination of execution after a timeout has passed.
    """

    def __init__(self, function, limit):
        """Initialize instance in preparation for being called."""
        self.__limit = limit
        self.__function = function
        self.__name__ = function.__name__
        self.__doc__ = function.__doc__
        self.__timeout = time.time()
        self.__process = multiprocessing.Process()
        self.__queue = multiprocessing.Queue()

    def __call__(self, *args, **kwargs):
        """Execute the embedded function object asynchronously.

        The function given to the constructor is transparently called and
        requires that "ready" be intermittently polled. If and when it is
        True, the "value" property may then be checked for returned data.
        """
        self.__limit = kwargs.pop('timeout', self.__limit)
        self.__queue = multiprocessing.Queue(1)
        args = (self.__queue, self.__function) + args
        self.__process = multiprocessing.Process(target=_target,
                                                 args=args,
                                                 kwargs=kwargs)
        self.__process.daemon = True
        self.__process.start()
        self.__timeout = self.__limit + time.time()
        while not self.ready:
            time.sleep(0.01)
        return self.value

    def cancel(self):
        """Terminate any possible execution of the embedded function."""
        if self.__process.is_alive():
            _killchildprocesses(self.__process.pid)
            self.__process.terminate()
        raise MyTimeoutError()

    @property
    def ready(self):
        """Read-only property indicating status of "value" property."""
        if self.__timeout < time.time():
            self.cancel()
        return self.__queue.full() and not self.__queue.empty()

    @property
    def value(self):
        """Read-only property containing data returned from function."""
        if self.ready is True:  # pragma: no cover
            flag, load = self.__queue.get()
            if flag:
                return load
            raise load  # pragma: no cover
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        test_testat_0.py

    FIRST RELEASE
        2016-07-06  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""

import pytest
from cuebung_helper import RunCode, ReferenceReturnObject

PROGRAM_NAME = 'testat-0'
IGNORE_COMPILER_WARNINGS = False


def int_arith_avg(args):
    """Reference implementation"""
    a, b = args
    ret = int((a + b) / 2)
    reference = ReferenceReturnObject(["%d" % (ret)])

    return reference

TESTCASES = [(0, 0), (1, 0), (42, 21), (1, 1), (3, 9), (4, 2), (-1120, 1110),
             (-24, -51)]


@pytest.fixture(params=TESTCASES, ids=[repr(tst) for tst in TESTCASES])
def args(request):
    """Iterate over all testcases"""
    return request.param


def test_testat_0(filename, args):
    """Test the c code and compare to reference implementation"""
    result = RunCode(filename, args, ['%d'] * len(args)).run(use_stdin=True)
    reference = int_arith_avg(args)
    assert reference.validate(result)

# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests"""

import os
from io import BytesIO
import pytest
from hypothesis import given, settings, Verbosity
import hypothesis.strategies as st

from cuebung_helper.helper import (_islistofstrings, _cmplistofstrings,
                                   _cleanresult, Boolean, Text,
                                   exchange_values)

__author__ = "Sebastian Stigler"
__copyright__ = "Sebastian Stigler"
__license__ = "mit"

settings.register_profile("ci", settings(max_examples=1000))
settings.register_profile("dev", settings(max_examples=10))
settings.register_profile("debug", settings(max_examples=10,
                                            verbosity=Verbosity.verbose))
settings.load_profile(os.getenv(u'HYPOTHESIS_PROFILE', 'default'))


@given(st.lists(st.text()))
def test_islistofstings(arg):
    """Test the function with the correct argument type"""
    assert _islistofstrings(arg)


NONETEXT = st.sampled_from([st.integers(), st.booleans(), st.floats()])


@given(st.lists(NONETEXT, min_size=1))
def test_not_islistofstings(arg):
    """Test the function with the incorrect argument type"""
    assert not _islistofstrings(arg)


@given(st.lists(NONETEXT, min_size=1))
def test_not_islistofstings2(arg):
    """Test the function with the incorrect argument type"""
    assert not _islistofstrings(tuple(arg))


@given(st.lists(st.text()))
def test_cmplistofstrings(arg):
    """Test the function with the correct argument type"""
    assert _cmplistofstrings(arg, arg)


@given(st.lists(st.text(), min_size=1))
def test_cmplistofstrings_diff_len(arg):
    """Test the function with two lists of not equal length"""
    assert not _cmplistofstrings(arg, arg[:-1])


@given(st.lists(st.text(), min_size=2),
       st.text())
def test_cmplistofstrings_diff_str(arg, swapword):
    """Test the function with two lists with different strings"""
    sndlst = arg[:]
    for i, word in enumerate(sndlst):
        if word != swapword:
            sndlst[i] = swapword
            break
    else:
        # if no entry in the list differs from swapword, force inequality
        # with a diffrent type of entry at the last place in the list
        sndlst[-1] = 42
    assert not _cmplistofstrings(arg, sndlst)


@given(st.lists(st.binary()))
def test_cleanresult(arg):
    """Test the function"""
    data = BytesIO(b'\n'.join(arg))
    out = _cleanresult(data)
    assert _islistofstrings(out)


@given(st.booleans())
def test_boolean(val):
    """Test the Boolean descriptor"""
    class UseBoolean:
        """Helper class"""
        val1 = Boolean(val)
        val2 = Boolean()

    use = UseBoolean()
    assert use.val1 == val
    assert not use.val2

    use.val1 = not val
    assert use.val1 != val
    use.val1 = val
    assert use.val1 == val

    # Test preset value
    assert not use.val2
    use.val2 = True
    assert use.val2
    use.val2 = False
    assert not use.val2

    with pytest.raises(ValueError):
        use.val2 = 1  # pylint: disable=R0204


@given(st.text(min_size=2))
def test_text(val):
    """Test the Text descriptor"""
    lval = int(len(val) / 2)
    sval1 = val[:lval]
    sval2 = val[lval:]

    class UseText:
        """Helper class"""
        val1 = Text(sval1)
        val2 = Text()

    use = UseText()
    assert use.val1 == sval1
    assert use.val2 is None

    use.val1 = sval2
    assert use.val1 == sval2
    use.val1 = sval1
    assert use.val1 == sval1

    # Test preset value
    assert use.val2 is None
    use.val2 = sval2
    assert use.val2 == sval2
    use.val2 = sval1
    assert use.val2 == sval1

    with pytest.raises(ValueError):
        use.val2 = 1


def test_mix_boolean_text():
    """Check if classes with mixed Descriptors preserve the default values"""

    class Mix():
        """Helper class"""
        val1 = Boolean()
        val2 = Text()
        val3 = Boolean()
        val4 = Text()

    use = Mix()
    assert not use.val1
    assert use.val2 is None
    assert not use.val3
    assert use.val4 is None
    assert isinstance(use.val1, bool)
    assert isinstance(use.val3, bool)
    use.val2 = "hallo"
    use.val4 = "du"
    assert isinstance(use.val2, str)
    assert isinstance(use.val4, str)


def test_boolean_without_class():
    """Test the getter"""
    val1 = Boolean()
    assert isinstance(val1, Boolean)


def test_exchange_values():
    """Test the exchanging of values via named pipes"""
    from random import randint
    valuelist = []
    with exchange_values(randint, (0, 50), valuelist):
        msg = os.open('message', os.O_RDONLY)
        cmd = os.open('command', os.O_WRONLY)
        innervalues = []
        for _ in range(10):
            os.write(cmd, b'N')
            val = os.read(msg, 10).decode()
            innervalues.append(int(val))
        os.close(msg)
        os.close(cmd)
    assert valuelist == innervalues
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



"""
    Dummy conftest.py for cuebung_helper.

    If you don't know what this is for, just leave it empty.
    Read more about conftest.py under:
    https://pytest.org/latest/plugins.html
"""
from __future__ import print_function, absolute_import, division

import pytest  # pylint: disable=W0611
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests"""

from collections import namedtuple
from io import BytesIO
import os
import pytest
from hypothesis import given, settings, Verbosity, example
import hypothesis.strategies as st

from cuebung_helper.emitter import set_emit, MsgType, reset_emit
from cuebung_helper.helper import _cmplistofstrings, _cleanresult, Config
from cuebung_helper.returnobject import (ReturnObject, PopenReturnObject,
                                         EmitMessages, ReferenceReturnObject)

__author__ = "Sebastian Stigler"
__copyright__ = "Sebastian Stigler"
__license__ = "mit"

settings.register_profile("ci", settings(max_examples=1000))
settings.register_profile("dev", settings(max_examples=10))
settings.register_profile("debug", settings(max_examples=10,
                                            verbosity=Verbosity.verbose))
settings.load_profile(os.getenv(u'HYPOTHESIS_PROFILE', 'default'))


@given(st.lists(st.text()),
       st.lists(st.text()),
       st.lists(st.text()),
       st.integers(min_value=0, max_value=255),
       st.booleans(),
       st.text())
def test_returnobject(stdout, stderr, error, returncode, timedout, msg):
    """ Test the object with legal data"""
    obj = ReturnObject(stdout, stderr, returncode, timedout, error)
    assert _cmplistofstrings(stdout, obj.stdout)
    assert _cmplistofstrings(stderr, obj.stderr)
    assert returncode == obj.returncode
    assert timedout == obj.timedout
    assert _cmplistofstrings(error, obj.error)

    txt = ("{c}(stdout={o}, stderr={e}, returncode={r}, timedout={t}, "
           "error={x})")
    filled_txt = txt.format(c='ReturnObject', o=stdout, e=stderr, r=returncode,
                            t=timedout, x=error)
    assert filled_txt == repr(obj)
    vgldict = {'msg': msg, 'stdout': stdout, 'stderr': stderr,
               'returncode': returncode, 'timedout': timedout, 'error': error}
    assert vgldict == obj.to_dict(msg)


@given(st.lists(st.integers(), min_size=1),
       st.integers(min_value=256))
def test_returnobject_illegal_input(tstlst, tstint):
    """ Test the object with illegal data"""
    with pytest.raises(ValueError):
        ReturnObject(stdout=tstlst)

    with pytest.raises(ValueError):
        ReturnObject(stderr=tstlst)

    with pytest.raises(ValueError):
        ReturnObject(returncode=tstint)

    with pytest.raises(ValueError):
        ReturnObject(returncode=-tstint + 255)

    with pytest.raises(ValueError):
        ReturnObject(returncode=tstlst)

    with pytest.raises(ValueError):
        ReturnObject(timedout=tstlst)

    with pytest.raises(ValueError):
        ReturnObject(error=tstlst)


@given(st.lists(st.text()),
       st.lists(st.text()),
       st.lists(st.text()),
       st.integers(min_value=0, max_value=255),
       st.booleans())
def test_returnobject_eq(stdout, stderr, error, returncode, timedout):
    """ Test equality and inequality"""
    obj = ReturnObject(stdout, stderr, returncode, timedout, error)
    vglobj = ReturnObject(stdout, stderr, returncode, timedout, error)
    assert obj == vglobj
    assert not obj != vglobj  # pylint: disable=C0113

    err_stdout = stdout[:]
    err_stdout.append("test")
    err_stderr = stderr[:]
    err_stderr.append("test")
    err_returncode = (returncode + 1) % 256
    err_timedout = not timedout
    err_error = error[:]
    err_error.append("test")
    assert obj != ReturnObject(err_stdout, stderr, returncode, timedout, error)
    assert obj != ReturnObject(stdout, err_stderr, returncode, timedout, error)
    assert obj != ReturnObject(stdout, stderr, err_returncode, timedout, error)
    assert obj != ReturnObject(stdout, stderr, returncode, err_timedout, error)
    assert obj != ReturnObject(stdout, stderr, returncode, timedout, err_error)
    with pytest.raises(TypeError):
        obj == 1  # pylint: disable=W0104


@given(st.lists(st.binary()),
       st.lists(st.binary()),
       st.integers(min_value=0, max_value=255),
       st.booleans())
def test_popenreturnobject_vgl(stdout, stderr, returncode, use_stdin):
    """ Test the object with legal data"""
    popen = namedtuple('Popen', 'stdout stderr returncode')
    b_stdout = b'\n'.join(stdout)
    b_stderr = b'\n'.join(stderr)
    result = popen(BytesIO(b_stdout), BytesIO(b_stderr), returncode)
    obj = PopenReturnObject.from_popen(result, cfg=Config(use_stdin, ' ', ' '))
    vglobj = ReturnObject(_cleanresult(BytesIO(b_stdout)),
                          _cleanresult(BytesIO(b_stderr)), returncode)
    assert vglobj == obj
    assert obj.cfg.use_stdin == use_stdin


@given(st.sampled_from([st.integers(), st.floats(), st.text()]),
       st.sampled_from([st.integers(), st.floats(), st.booleans()]))
def test_popenreturnobject_illegal(use_stdin, command):
    """Test the object with additional illegal input"""

    with pytest.raises(ValueError):
        PopenReturnObject(cfg=use_stdin)

    with pytest.raises(ValueError):
        PopenReturnObject(cfg=Config(use_stdin, 'NA', 'NA'))

    with pytest.raises(ValueError):
        PopenReturnObject(cfg=Config(True, command, 'NA'))

    with pytest.raises(ValueError):
        PopenReturnObject(cfg=Config(True, 'NA', command))


@given(st.text(),
       st.text(),
       st.integers(),
       st.booleans())
def test_emitmessages(stdin_txt, commandline_arg_txt, error_txt, use_stdin):
    """Test the emitmessage object"""
    emt = EmitMessages()

    with pytest.raises(ValueError):
        emt.command_templ = error_txt

    emt.use_stdin = use_stdin

    if emt.use_stdin:
        emt.command_templ = stdin_txt
        assert emt.command_templ == stdin_txt
    else:
        emt.command_templ = commandline_arg_txt
        assert emt.command_templ == commandline_arg_txt


def myemit(msg):
    """help function"""
    global RETURNVALUE  # pylint: disable=W0603
    RETURNVALUE = msg

RETURNVALUE = None


@given(st.sampled_from([st.text(), None]))
def test_rro_validate_basic(msg_extra):
    """Simple validation test"""
    obj = PopenReturnObject()
    reference = ReferenceReturnObject()

    set_emit(myemit)
    assert reference.validate(obj, msg_extra)
    reset_emit()

    assert RETURNVALUE['msgtype'] == MsgType.passed
    assert RETURNVALUE['msg'] == reference.txt.passed.general
    if msg_extra is not None:
        assert RETURNVALUE['msg_extra'] == msg_extra


def test_rro_validate_invalide_args():
    """Test for invalid object"""

    obj = ReturnObject()
    reference = ReferenceReturnObject()

    with pytest.raises(TypeError):
        reference.validate(obj)

    with pytest.raises(TypeError):
        reference.validate(None)


@given(st.binary(min_size=1),
       st.binary(min_size=1),
       st.binary(min_size=1),
       st.integers(min_value=1, max_value=255),
       st.booleans())
@example(b'\n', b'\x00', b'\x00', 1, False)
def test_rro_validate_not_eq(stdout, stderr, error, returncode, use_stdin):
    """ Test the object with differnt results"""
    popen = namedtuple('Popen', 'stdout stderr returncode')
    stdout += b'x'
    stderr += b'x'
    error += b'x'
    result = popen(BytesIO(stdout), BytesIO(stderr), returncode)
    cfg = Config(use_stdin, ' ', ' ')
    obj = PopenReturnObject.from_popen(result, error, cfg)
    obj._timedout = True  # pylint: disable=W0212

    reference = ReferenceReturnObject()

    set_emit(myemit)
    # timedout
    assert not reference.validate(obj)
    assert RETURNVALUE['msgtype'] == MsgType.failed_timedout
    reference.dont_check.timedout = True
    # returncode
    assert not reference.validate(obj)
    assert RETURNVALUE['msgtype'] == MsgType.failed_returncode
    reference.dont_check.returncode = True
    # output_linecount
    assert not reference.validate(obj)
    assert RETURNVALUE['msgtype'] == MsgType.failed_output_linecount
    reference.dont_check.output_linecount = True
    # stderr
    assert not reference.validate(obj)
    assert RETURNVALUE['msgtype'] == MsgType.failed_stderr
    reference.dont_check.stderr = True
    # stdout
    assert not reference.validate(obj)
    print(RETURNVALUE)
    assert RETURNVALUE['msgtype'] == MsgType.failed_stdout
    reference.dont_check.stdout = True
    # error
    assert not reference.validate(obj)
    assert RETURNVALUE['msgtype'] == MsgType.failed_error
    reference.dont_check.error = True
    # general
    assert not reference.validate(obj)
    assert RETURNVALUE['msgtype'] == MsgType.failed
    reset_emit()
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests"""

import builtins
import os
from unittest.mock import patch
from hypothesis import given, settings, Verbosity
import hypothesis.strategies as st

from cuebung_helper.emitter import emit, set_emit, reset_emit, MsgType
import cuebung_helper.emitter

__author__ = "Sebastian Stigler"
__copyright__ = "Sebastian Stigler"
__license__ = "mit"

settings.register_profile("ci", settings(max_examples=1000))
settings.register_profile("dev", settings(max_examples=10))
settings.register_profile("debug", settings(max_examples=10,
                                            verbosity=Verbosity.verbose))
settings.load_profile(os.getenv(u'HYPOTHESIS_PROFILE', 'default'))


def my_emit(msg):
    """Custom emit"""
    return msg

cuebung_helper.emitter.print = builtins.print


@given(st.text())
def test_emit(arg):
    """Test the function with the correct argument type"""
    msg = {'msg': arg}
    msg_result = msg.copy()
    msg_result['msgtype'] = MsgType.message
    with patch('cuebung_helper.emitter.print', autospec=True) as mock_print:
        emit(msg)
        mock_print.assert_called_with(msg_result)


@given(st.text())
def test_set_emit(arg):
    """Test set_emit"""
    reset_emit()
    set_emit(my_emit)
    msg = {'msg': arg}
    msg_result = msg.copy()
    msg_result['msgtype'] = MsgType.message
    assert emit(msg) == msg_result


# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        runcode.py -- Run c code

    FIRST RELEASE
        2016-07-04  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""
import errno
import os
from shlex import quote
from subprocess import Popen, PIPE
from cuebung_helper import __version__
from .timeout_decorator import timeout as mytimeout, MyTimeoutError
from .returnobject import PopenReturnObject, ReferenceReturnObject
from .helper import _islistofstrings, Command, Config
from .emitter import emit, MsgType

__all__ = ['RunCode', 'Command', 'Make']
__author__ = "Sebastian Stigler"
__copyright__ = "Sebastian Stigler"
__license__ = "mit"

BLOCK_SIZE = 100000


class RunCode():
    """Run c code in a subprocess"""

    def __init__(self, command, args=None, formatlist=None, timeout=5,
                 fifo='error'):
        if isinstance(command, Command):
            self._command = command
        else:
            self._command = Command('', command)
        self._args = args or ()
        self._formatlist = formatlist or []
        self._timeout = timeout
        self._fifo = fifo
        self._fifofd = None
        self.checkrep()
        # May fail with TypeError if the args and argsstring don't fit together
        self._argsstringlist = [fstr % arg for fstr, arg in
                                zip(self.formatlist, self.args)]

    @property
    def cmd(self):
        """Getter for cmd"""
        return ' '.join(self.command).strip()

    @property
    def command(self):
        """Getter for command"""
        return self._command

    @property
    def args(self):
        """Getter for args"""
        return self._args

    @property
    def formatlist(self):
        """Getter for formatlist"""
        return self._formatlist

    @property
    def argsstring(self):
        """Getter for argsstring"""
        return " ".join(self._argsstringlist)

    @property
    def argsstringlist(self):
        """Getter for argsstring"""
        return self._argsstringlist

    @property
    def quoted_argsstring(self):
        """Getter for quoted argssting"""
        return ' ' + ' '.join(quote(arg) for arg in self.argsstringlist)

    @property
    def timeout(self):
        """Getter for timeout"""
        return self._timeout

    @property
    def fifo(self):
        """Getter for fifo"""
        return self._fifo

    def _cfg(self, use_stdin):
        """Get config  for PopenReturnObject"""
        if use_stdin:
            return Config(use_stdin, self.command.filename, self.argsstring)
        else:
            return Config(use_stdin, self.command.filename,
                          self.quoted_argsstring)

    def checkrep(self):
        """Consistency check"""
        if not isinstance(self.command.ldpreload, str):
            raise ValueError('command.ldpreload must be a string')
        if not (len(self.command.ldpreload) == 0 or
                self.command.ldpreload.startswith('LD_PRELOAD=')):
            raise ValueError('command.ldpreload must be empty or start with '
                             '"LD_PRELOAD="')
        if not (isinstance(self.command.filename, str) and
                len(self.command.filename) > 0):
            raise ValueError('command.filename must be a none empty string')
        if not isinstance(self.args, tuple):
            raise ValueError('args has to be a tuple of values or None')
        if not _islistofstrings(self.formatlist):
            raise ValueError('formatlist must be a list of formatstrings')
        if not all(entry.startswith('%') for entry in self.formatlist):
            raise ValueError('formatlist must be a string composed of '
                             'space separated formatstrings like "%s" "%d" '
                             'and "%f" (including padding and truncation)')
        if len(self.formatlist) != len(self.args):
            raise ValueError('formatlist and args must be of equal length')
        if not (isinstance(self.timeout, (int, float)) and self.timeout >= 0):
            raise ValueError('timeout must be an positiv number')
        if not isinstance(self.fifo, str) or len(self.fifo) < 1:
            raise ValueError('fifo mut be a valid filename for a named pipe')

    def _prepare_named_pipe(self):
        """Prepare a new named pipe

        :return: a read and non blocking file descripter to the fifo
        """
        if os.path.exists(self.fifo):
            os.unlink(self.fifo)  # pragma: no cover
        if not os.path.exists(self.fifo):  # pragma: no cover
            os.mkfifo(self.fifo, 0o600)
        self._fifofd = os.open(self.fifo, os.O_RDONLY | os.O_NONBLOCK)

    def _fetch_named_pipe(self):
        """Fetch data from named pipe

        :return: bytestring
        """
        try:
            error = os.read(self._fifofd, BLOCK_SIZE)
        except IOError as err:  # pragma: no cover
            if (err.errno == errno.EAGAIN or  # pragma: no cover
                    err.errno == errno.EWOULDBLOCK):  # pragma: no cover
                error = b''  # pragma: no cover
            else:  # pragma: no cover
                raise  # pragma: no cover
        return error

    def _cleanup_named_pipe(self):
        """Cleanup named pipe"""
        os.close(self._fifofd)
        if os.path.exists(self.fifo):  # pragma: no cover
            os.unlink(self.fifo)

    def _run(self, use_stdin):
        """Run the command and store the result in a PopenReturnObject

        :param use_stdin: stream the argument via stdin to the program or pass
            the arguments as commandline parameters.

        :return: a `PopenReturnObject`
        """
        cmd = self.cmd
        if not use_stdin:
            cmd += self.quoted_argsstring
        res = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE, stdin=PIPE,
                    bufsize=0)
        if use_stdin:
            argsstring = self.argsstring + '\n'
            res.stdin.write(argsstring.encode())
        res.wait()
        error = self._fetch_named_pipe()
        return PopenReturnObject.from_popen(res, error, self._cfg(use_stdin))

    def run(self, use_stdin=False):
        """Run the command under the timeout constraint"""
        self._prepare_named_pipe()
        try:
            ret = mytimeout(self.timeout, False)(self._run)(use_stdin)
        except MyTimeoutError:
            ret = PopenReturnObject(timedout=True, cfg=self._cfg(use_stdin))
        finally:
            self._cleanup_named_pipe()
        return ret

    def __repr__(self):
        txt = "{n}(cmd={c}, args={a}, formatlist={s}, timeout={t}, fifo={f})"
        return txt.format(n=self.__class__.__name__, c=self.cmd, a=self.args,
                          s=self.formatlist, t=self.timeout, f=self.fifo)


class Make():
    """Wrapper for make, make clean and make test"""

    txt_clean_err = 'Ausführung von "make clean" ist fehlgeschlagen.'
    txt_timeout = 'Das Kompilieren dauert zu lange.'
    txt_errors = 'Es sind Kompiler-Fehler aufgetreten.'
    txt_warnings = 'Es sind Kompiler Warnungen aufgetreten.'
    txt_ok = 'Das Kompilieren war erfolgreich.'

    def __init__(self, ignore_warnings=False, timeout=30):
        self.timeout = timeout
        self.ignore_warnings = ignore_warnings

    def clean(self):
        """Run make clean, check result and emit messages"""
        result = RunCode('make', ('clean',), ['%s'],
                         timeout=self.timeout).run()
        if result.timedout or result.returncode != 0:
            reference = ReferenceReturnObject()
            msg = reference._merge(result)  # pylint: disable=W0212
            msg['msg_extra'] = 'make clean'
            msg['msg'] = self.txt_clean_err
            emit(msg, msgtype=MsgType.failed)
            return False
        return True

    def test(self):
        """Run make test, check result and emit messages"""
        result = RunCode('make', ('test',), ['%s'], timeout=self.timeout).run()
        return self.checkresult(result, 'make test')

    def all(self):
        """Run make all, check result and emit messages"""
        result = RunCode('make', ('all',), ['%s'], timeout=self.timeout).run()
        return self.checkresult(result, 'make all')

    def checkresult(self, result, msg_extra):
        """check result and emit message"""
        reference = ReferenceReturnObject()
        msg = reference._merge(result)  # pylint: disable=W0212
        msg['msg_extra'] = msg_extra
        cls = self.__class__

        if result.timedout:
            msg['msg'] = cls.txt_timeout
            emit(msg, msgtype=MsgType.failed_timedout)
            return False
        if result.returncode != 0:
            msg['msg'] = cls.txt_errors
            emit(msg, msgtype=MsgType.failed_returncode)
            return False
        if result.stderr != []:
            msg['msg'] = cls.txt_warnings
            emit(msg, msgtype=MsgType.failed_stderr)
            return self.ignore_warnings
        else:
            msg['msg'] = cls.txt_ok
            emit(msg, msgtype=MsgType.passed)
            return True

# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests"""

import unittest.mock as mock
import pytest

from cuebung_helper import Make, PopenReturnObject, set_emit

__author__ = "Sebastian Stigler"
__copyright__ = "Sebastian Stigler"
__license__ = "mit"


def myemit(msg):
    """help function"""
    global RETURNVALUE  # pylint: disable=W0603
    RETURNVALUE = msg

RETURNVALUE = None


@pytest.fixture(params=[False, True])
def ign_comp_warn(request):
    """Use signals for timing out or not."""
    return request.param


@pytest.fixture(params=['all', 'test'])
def func(request):
    """Use signals for timing out or not."""
    return request.param


@mock.patch('cuebung_helper.runcode.RunCode.run')
def test_make_clean(mock_runcode, ign_comp_warn):
    """Test Make class with clean methode"""
    res_ok = PopenReturnObject()
    res_to = PopenReturnObject(timedout=True)
    res_rc = PopenReturnObject(returncode=1)
    set_emit(myemit)

    make = Make(ignore_warnings=ign_comp_warn)

    mock_runcode.return_value = res_ok
    assert make.clean()
    assert mock_runcode.called

    mock_runcode.return_value = res_to
    assert not make.clean()
    assert RETURNVALUE['msg'] == Make.txt_clean_err

    mock_runcode.return_value = res_rc
    assert not make.clean()
    assert RETURNVALUE['msg'] == Make.txt_clean_err


@mock.patch('cuebung_helper.runcode.RunCode.run')
def test_make_all_test(mock_runcode, ign_comp_warn, func):
    """Test Make class with all/test methode"""
    res_ok = PopenReturnObject()
    res_to = PopenReturnObject(timedout=True)
    res_rc = PopenReturnObject(returncode=1)
    res_err = PopenReturnObject(stderr=['XYZ'])
    set_emit(myemit)

    make = Make(ignore_warnings=ign_comp_warn)

    mock_runcode.return_value = res_ok
    assert getattr(make, func)()
    assert RETURNVALUE['msg'] == Make.txt_ok

    mock_runcode.return_value = res_to
    assert not getattr(make, func)()
    assert RETURNVALUE['msg'] == Make.txt_timeout

    mock_runcode.return_value = res_rc
    assert not getattr(make, func)()
    assert RETURNVALUE['msg'] == Make.txt_errors

    mock_runcode.return_value = res_err
    assert ign_comp_warn == getattr(make, func)()
    assert RETURNVALUE['msg'] == Make.txt_warnings
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        conftest.py

    FIRST RELEASE
        2016-07-06  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""
import json
import os.path
import pytest
from cuebung_helper import Make, Command, emit, set_emit

DIRNAME = os.path.dirname(os.path.realpath(__file__))
LIBDIR = os.path.realpath(os.path.join(DIRNAME, '..', 'lib'))

LD_PRELOAD = []
LD_PRELOAD.append(os.path.join(LIBDIR, 'emit.so'))
GFORBIDDEN = os.environ.get('GENERAL_FORBIDDEN', None)
if GFORBIDDEN is not None:
    LD_PRELOAD.append(GFORBIDDEN)
LD_PRELOAD.append(os.path.join(DIRNAME, "forbidden.so"))


def my_emit(msg):
    """Write msg as json dump to file"""
    assert isinstance(msg, dict)
    with open('output', 'a') as outputfile:
        outputfile.write(json.dumps(msg) + '\n')

set_emit(my_emit)


@pytest.fixture(scope="module")
def filename(request):
    """Fixture compiles the c program and do some prechecks"""
    program = getattr(request.module, 'PROGRAM_NAME', 'cmd')
    ignore_warnings = getattr(request.module, 'IGNORE_COMPILER_WARNINGS',
                              False)
    no_ld_pre = getattr(request.module, 'NO_LD_PRELOAD', False)
    make = Make(ignore_warnings=ignore_warnings, timeout=30)
    assert make.clean()
    assert make.test()
    fname = os.path.join(DIRNAME, program)
    if not os.path.isfile(fname):
        msg = {'msg': 'Das Program "%s" wurde nicht erstellt.' % (program)}
        emit(msg)
        assert False

    def fin():
        """finalizer"""
        emit({'msg': 'Alle Tests durchlaufen'})

    request.addfinalizer(fin)
    if no_ld_pre:
        ldp = ''
    else:
        ldp = 'LD_PRELOAD=%s' % (':'.join(LD_PRELOAD))
    return Command(ldp, fname)


# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        conftest.py

    FIRST RELEASE
        2016-07-28  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""
import pytest
from magic import Magic, MagicException

DEFAULT_EXT = '.py,.md'
DEFAULT_ENC = 'us-ascii,utf-8'


def pytest_addoption(parser):
    """Define options"""
    group = parser.getgroup("general")
    group.addoption('--checkencoding', action='store_true',
                    help="check files for the correct encoding")
    parser.addini("ce_ignore", type="linelist",
                  help="each line specifies a glob pattern of files to ignore")
    parser.addini("ce_ext",
                  help="comma separated extentions (default %s)" % DEFAULT_EXT)
    parser.addini("ce_encoding",
                  help="comma separated encodings (default %s)" % DEFAULT_EXT)


def pytest_sessionstart(session):
    """Initialite some additional attributes of config"""
    config = session.config
    if config.option.checkencoding:
        config.ce_ignore = Ignorer(config.getini("ce_ignore"))
        config.ce_ext = config.getini("ce_ext") or DEFAULT_EXT
        config.ce_encoding = config.getini("ce_encoding") or DEFAULT_ENC


def pytest_collect_file(parent, path):
    """Collect files to test for correct encoding"""
    config = parent.config
    if not config.option.checkencoding:
        return
    if path.ext in config.ce_ext.split(','):
        ignore = config.ce_ignore(path)
        if not ignore:
            return EncodingFile(path, parent)


class EncodingFile(pytest.File):  # pylint: disable=E1101
    """EncodingFile"""

    def collect(self):
        """Detect the encoding of the file"""
        blob = self.fspath.open('rb').read()
        magic = Magic(mime_encoding=True)
        try:
            encoding = magic.from_buffer(blob)
        except MagicException:
            self.add_marker('skip')
            encoding = None
        path = self.fspath
        ppath = self.parent.fspath
        rel_path = path.strpath.replace(ppath.strpath, '', 1)[1:]
        spec = (rel_path, encoding)
        enc = self.parent.config.ce_encoding
        yield EncodingItem('Check file encoding [%s]' % enc, self, spec)


class EncodingItem(pytest.Item):  # pylint: disable=E1101
    """EncodingItem"""

    def __init__(self, name, parent, spec):
        super(EncodingItem, self).__init__(name, parent)
        self.spec = spec
        self.add_marker('encoding')

    def runtest(self):
        """Do the actual encoding check"""
        config = self.parent.config
        if self.spec[1].lower() not in config.ce_encoding.split(','):
            raise EncodingException(self, self.spec)

    def repr_failure(self, excinfo):
        """ called when self.runtest() raises an exception. """
        enc = self.parent.config.ce_encoding
        if isinstance(excinfo.value, EncodingException):
            return "\n".join([
                "Wrong encoding detected. Expacted encoding in [%s]." % enc,
                "   Detected encoding of %r: %r" % excinfo.value.args[1]
            ])
        else:
            return "%r" % excinfo.value

    def reportinfo(self):
        """Infotest on failure"""
        return self.fspath, 1, "usecase: %s %s" % (self.name, self.spec)


class EncodingException(Exception):
    """ custom exception for error reporting. """


class Ignorer:
    """Ignorer Class"""
    def __init__(self, ignorelines):
        self.ignores = ignores = []
        for line in ignorelines:
            i = line.find("#")
            if i != -1:
                line = line[:i]
            ignores.append(line)

    def __call__(self, path):
        for glob in self.ignores:
            if path.fnmatch(glob):
                return True
        return False
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



"""Timeout decorator tests."""
import time

import pytest

from cuebung_helper.timeout_decorator import timeout, MyTimeoutError


@pytest.fixture(params=[False, True])
def use_signals(request):
    """Use signals for timing out or not."""
    return request.param


def test_timeout_decorator_arg(use_signals):
    """Test"""
    @timeout(1, use_signals=use_signals)
    def func():
        """Test function"""
        time.sleep(2)
    with pytest.raises(MyTimeoutError):
        func()


def test_timeout_kwargs(use_signals):
    """Test"""
    @timeout(3, use_signals=use_signals)
    def func():
        """Test function"""
        time.sleep(2)
    with pytest.raises(MyTimeoutError):
        func(timeout=1)  # pylint: disable=E1123


def test_timeout_no_seconds(use_signals):
    """Test"""
    @timeout(use_signals=use_signals)
    def func():
        """Test function"""
        time.sleep(0.1)
    func()


def test_timeout_partial_seconds(use_signals):
    """Test"""
    @timeout(0.2, use_signals=use_signals)
    def func():
        """Test function"""
        time.sleep(0.5)
    with pytest.raises(MyTimeoutError):
        func()


def test_timeout_ok(use_signals):
    """Test"""
    @timeout(seconds=2, use_signals=use_signals)
    def func():
        """Test function"""
        time.sleep(1)
    func()


def test_function_name(use_signals):
    """Test"""
    @timeout(seconds=2, use_signals=use_signals)
    def func_name():
        """Test function"""
        pass

    assert func_name.__name__ == 'func_name'


def test_timeout_pickle_error():
    """Test that when a pickle error occurs a timeout error is raised."""
    @timeout(seconds=1, use_signals=False)
    def func():
        """Test function"""
        time.sleep(0.1)

        class Test(object):
            """Test class"""
            pass
        return Test()
    with pytest.raises(MyTimeoutError):
        func()
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Tests"""

import os
import pytest
from hypothesis import given, settings, Verbosity
import hypothesis.strategies as st

from cuebung_helper import RunCode
from cuebung_helper.helper import Command

__author__ = "Sebastian Stigler"
__copyright__ = "Sebastian Stigler"
__license__ = "mit"

settings.register_profile("ci", settings(max_examples=1000))
settings.register_profile("dev", settings(max_examples=10))
settings.register_profile("debug", settings(max_examples=10,
                                            verbosity=Verbosity.verbose))
settings.load_profile(os.getenv(u'HYPOTHESIS_PROFILE', 'default'))

BASEPATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'scripts')


@given(st.lists(st.integers()),
       st.integers(min_value=0))
def test_runcode(args, timeout):
    """Basic test for RunCode"""
    cmd = os.path.join(BASEPATH, 'emptytest')
    formatlist = ['%d'] * len(args)
    if len(args) == 0:
        args = None
        vglargs = ()
        formatlist = None
        vglfmtlist = []
    else:
        args = tuple(args)
        vglargs = args
        vglfmtlist = formatlist
    txt = "{n}(cmd={c}, args={a}, formatlist={s}, timeout={t}, fifo={f})"
    filled = txt.format(n='RunCode', c=cmd, a=vglargs, s=vglfmtlist, t=timeout,
                        f='error')
    assert repr(RunCode(cmd, args, formatlist, timeout)) == filled


@given(st.integers(),
       st.lists(st.integers(), min_size=1),
       st.integers(max_value=-1))
def test_runcode_illegal_args(cmd, args, timeout):
    """Test the class with illegal paramertes"""
    with pytest.raises(ValueError):
        RunCode(cmd)

    with pytest.raises(ValueError):
        RunCode('')

    with pytest.raises(ValueError):
        RunCode(' ', args, ['%d'] * len(args))

    with pytest.raises(ValueError):
        RunCode(' ', tuple(args), "")

    with pytest.raises(ValueError):
        RunCode(' ', tuple(args), args)

    with pytest.raises(ValueError):
        RunCode(' ', tuple(args), ['Xd'] * len(args))

    with pytest.raises(ValueError):
        RunCode(' ', tuple(args), ['%d'] * (len(args) - 1))

    with pytest.raises(ValueError):
        RunCode(' ', tuple(args), ['%d'] * len(args), timeout)

    with pytest.raises(ValueError):
        RunCode(' ', tuple(args), ['%ü'] * len(args))

    with pytest.raises(TypeError):
        RunCode(' ', tuple(str(arg) for arg in args), ['%d'] * len(args))

    with pytest.raises(ValueError):
        RunCode(' ', fifo='')

    with pytest.raises(ValueError):
        RunCode(' ', fifo=cmd)

    with pytest.raises(ValueError):
        RunCode(Command(cmd, cmd))

    with pytest.raises(ValueError):
        RunCode(Command('xxx', ''))


@given(st.tuples(st.integers(), st.integers()))
def test_runcode_stdin(args):
    """Test the class in the use_stdin mode with valid entries"""
    cmd = os.path.join(BASEPATH, 'run_stdin.py')
    formatlist = ['%d'] * 2
    timeout = 1
    runcode = RunCode(cmd, args, formatlist, timeout)
    ret = runcode.run(use_stdin=True)
    assert not ret.timedout
    assert ret.returncode == 3
    assert ret.stderr == [str(args[1])]
    assert ret.stdout == [str(args[0])]


@given(st.tuples(st.integers(), st.integers()))
def test_runcode_commandline(args):
    """Test the class in the commandline argument mode with valid entries"""
    cmd = os.path.join(BASEPATH, 'run_commandline.py')
    formatlist = ['%d'] * 2
    timeout = 1
    runcode = RunCode(cmd, args, formatlist, timeout)
    ret = runcode.run(use_stdin=False)
    assert not ret.timedout
    assert ret.returncode == 4
    assert ret.stderr == [str(args[1])]
    assert ret.stdout == [str(args[0])]


@given(st.integers(min_value=2))
def test_runcode_timeout(timeout):
    """Test the timeout constraint"""
    cmd = 'sleep'
    args = (timeout, )
    formatlist = ['%d']
    runcode = RunCode(cmd, args, formatlist, timeout=0.1)
    ret = runcode.run()
    assert ret.timedout


@given(st.tuples(st.integers(), st.integers(), st.integers()))
def test_runcode_namedpipe(args):
    """Test the class in the use_stdin mode with valid entries"""
    cmd = os.path.join(BASEPATH, 'run_named_pipe.py')
    formatlist = ['%d'] * 3
    timeout = 1
    runcode = RunCode(cmd, args, formatlist, timeout, fifo='/tmp/namedpipe')
    ret = runcode.run(use_stdin=True)
    assert not ret.timedout
    assert ret.returncode == 3
    assert ret.stderr == [str(args[1])]
    assert ret.stdout == [str(args[0])]
    assert ret.error == [str(args[2])]
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        returnobject.py -- ReturnObjects

    FIRST RELEASE
        2016-06-29  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""
from os.path import basename
from cuebung_helper import __version__
from .helper import (_islistofstrings, _cmplistofstrings, _cleanresult,
                     _lob2los, Boolean, Text, Config)
from .emitter import emit, MsgType

__author__ = "Sebastian Stigler"
__copyright__ = "Sebastian Stigler"
__license__ = "mit"
__all__ = ['PopenReturnObject', 'ReferenceReturnObject']


class ReturnObject():
    """Object to wrap the complete return stat of a systemcall"""

    def __init__(self, stdout=None, stderr=None, returncode=0, timedout=False,
                 error=None):
        self._stdout = stdout or []
        self._stderr = stderr or []
        self._returncode = returncode
        self._timedout = timedout
        self._error = error or []
        self.checkrep()

    @property
    def stdout(self):
        """Getter for stdout"""
        return self._stdout

    @property
    def stderr(self):
        """Getter for stderr"""
        return self._stderr

    @property
    def returncode(self):
        """Getter for returncode"""
        return self._returncode

    @property
    def timedout(self):
        """Getter for timedout"""
        return self._timedout

    @property
    def error(self):
        """Getter for error"""
        return self._error

    def checkrep(self):
        """Check the internal representation of the entries"""
        if not _islistofstrings(self.stdout):
            raise ValueError("stdout must be a list of strings.")
        if not _islistofstrings(self.stderr):
            raise ValueError("stderr must be a list of strings.")
        if not (isinstance(self.returncode, int) and
                0 <= self.returncode <= 255):
            raise ValueError("returncode is not an integer or the value is "
                             "not in [0, 255].")
        if not isinstance(self.timedout, bool):
            raise ValueError("timedout is not a bool.")
        if not _islistofstrings(self.error):
            raise ValueError("error must be a list of strings.")

    def __repr__(self):
        txt = ("{c}(stdout={o}, stderr={e}, returncode={r}, timedout={t}, "
               "error={x})")
        return txt.format(c=self.__class__.__name__, o=self.stdout,
                          e=self.stderr, r=self.returncode, t=self.timedout,
                          x=self.error)

    def __eq__(self, other):
        """Check if two ReturnObjects are equal"""
        if not isinstance(other, ReturnObject):
            raise TypeError('incompatible types %s == %s' % (type(self),
                                                             type(other)))
        if not _cmplistofstrings(self.stdout, other.stdout):
            return False
        if not _cmplistofstrings(self.stderr, other.stderr):
            return False
        if self.returncode != other.returncode:
            return False
        if self.timedout != other.timedout:
            return False
        if not _cmplistofstrings(self.error, other.error):
            return False
        return True

    def __ne__(self, other):
        """Check if two ReturnObjects are not equal"""
        return not self.__eq__(other)

    def to_dict(self, msg=None):
        """Return a dict"""
        return {'msg': msg, 'stdout': self.stdout, 'stderr': self.stderr,
                'returncode': self.returncode, 'timedout': self.timedout,
                'error': self.error}


class PopenReturnObject(ReturnObject):
    """Parse the output of an Popen call to an ReturnObject"""
    def __init__(self, stdout=None, stderr=None, returncode=0, timedout=False,
                 error=None, cfg=None):
        self._cfg = cfg or Config(False, 'N/A', 'N/A')
        super().__init__(stdout, stderr, returncode, timedout, error)

    @property
    def cfg(self):
        """Getter for cfg"""
        return self._cfg

    @property
    def use_stdin(self):
        """Getter for use_stdin"""
        return self._cfg.use_stdin

    @property
    def command(self):
        """Getter for command"""
        return self._cfg.command

    @property
    def args(self):
        """Getter for args"""
        return self._cfg.args

    def checkrep(self):
        """extedned version"""
        super().checkrep()
        if not isinstance(self.cfg, Config):
            raise ValueError('cfg must be a Config object')
        if not isinstance(self.use_stdin, bool):
            raise ValueError('cfg.use_stdin has to be a bool')
        if not isinstance(self.command, str) or len(self.cfg.command) == 0:
            raise ValueError('cfg.command has to be a none empty string')
        if not isinstance(self.args, str):
            raise ValueError('cfg.args has to be a string')

    def __repr__(self):
        txt = super().__repr__()[:-1]
        txt += ', cfg={c})'.format(c=self.cfg)
        return txt

    @classmethod
    def from_popen(cls, result, error=b'', cfg=None):
        """Create a new RetrunObject from the result of a subprocess.popen
        call"""
        stdout = _cleanresult(result.stdout)
        stderr = _cleanresult(result.stderr)
        error = _lob2los(error.split(b'\n'))
        return cls(stdout, stderr, result.returncode, False, error, cfg)


class ValidationConfig:
    """Config class for the valiation of your RunCode output with the Reference
    """
    timedout = Boolean()
    returncode = Boolean()
    output_linecount = Boolean()
    stderr = Boolean()
    stdout = Boolean()
    error = Boolean()


class EmitMessages:
    """Text for the validaton of your RunCode output"""
    class Passed:
        """Text for passed tests"""
        general = "Test bestanden."

    class Failed:
        """Text for failed tests"""
        general = Text("Test nicht bestanden.")
        timedout = Text("Test wurde abgebrochen, da er zu lange gedauert hat.")
        returncode = Text("Das Programm liefert den falschen Returncode.")
        output_linecount = Text("Die Ausgabe ist zu lang oder zu kurz.")
        stderr = Text("Die stderr Ausgabe ist inkorrekt.")
        stdout = Text("Die stdout Ausgabe ist inkorrekt.")
        error = Text("Es wurden Befehle benutzt, die nicht erlaubt sind.")

    class Template:
        """Template text for the command"""
        commandline_arg = Text("{command} {args}")
        stdin = Text('echo "{args}" | {command}')

    passed = Passed()
    failed = Failed()
    use_stdin = Boolean()
    _templ = Template()

    @property
    def command_templ(self):
        """Getter for the command template"""
        if self.use_stdin:
            return self._templ.stdin
        else:
            return self._templ.commandline_arg

    @command_templ.setter
    def command_templ(self, value):
        """Setter for the command template"""
        if isinstance(value, str):
            if self.use_stdin:
                self._templ.stdin = value
            else:
                self._templ.commandline_arg = value
        else:
            raise ValueError('value must be a string')


class ReferenceReturnObject(ReturnObject):
    """Implements some specific compare functions which emits custom
    messages"""

    def __init__(self, *args, **kwargs):
        self.dont_check = ValidationConfig()
        self.txt = EmitMessages()
        super().__init__(*args, **kwargs)

    def validate(self, other, msg_extra=None):
        """Compare the `other` PopenReturnObject with this one

        Will emit some messages if there are differences in the two objects

        :param other: a `PopenReturnObject` instance
        :param msg_extra: a costum message for the to sent dict.

        :return: True iff  self == other
                 False iff self != other
        """

        if not isinstance(other, PopenReturnObject):
            raise TypeError('validate only results of RunCode')

        msg = self._merge(other)

        if msg_extra is not None:
            msg['msg_extra'] = msg_extra

        self.txt.use_stdin = other.use_stdin
        command = self.txt.command_templ
        msg['command'] = command.format(  # pylint: disable=E1101
            command=basename(other.command),
            args=other.args)
        for event in ['timedout', 'returncode', 'output_linecount', 'stderr',
                      'stdout', 'error']:
            if len(msg[event]) == 2:
                if not getattr(self.dont_check, event):
                    msg['msg'] = getattr(self.txt.failed, event)
                    emit(msg, msgtype=MsgType['failed_' + event])
                    return False
        if self != other:
            msg['msg'] = self.txt.failed.general
            emit(msg, msgtype=MsgType.failed)
            return False

        msg['msg'] = self.txt.passed.general
        emit(msg, msgtype=MsgType.passed)
        return True

    def _merge(self, other):
        """Merge the dict of self and other

        Add a `result` subkey to the <key> entry if the <key> entries of self
        and other are not equal.

        :param other: a `PopenReturnObject` instance
        :return: `dict` with merged entries
        """
        self_dict = self.to_dict()
        other_dict = other.to_dict()
        result = {}
        txt_olc = 'output_linecount'
        result[txt_olc] = {'reference': 0}
        result_lc = 0
        for key in other_dict.keys():
            if key.startswith('msg'):
                continue
            result[key] = {}
            result[key]['reference'] = self_dict[key]
            if isinstance(other_dict[key], list):
                if key in ['stdout', 'stderr']:
                    result[txt_olc]['reference'] += len(self_dict[key])
                    result_lc += len(other_dict[key])
                if not _cmplistofstrings(self_dict[key], other_dict[key]):
                    result[key]['result'] = other_dict[key]
            else:
                if self_dict[key] != other_dict[key]:
                    result[key]['result'] = other_dict[key]
        if result_lc != result[txt_olc]['reference']:
            result[txt_olc]['result'] = result_lc
        return result

# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        run_commandline.py -- Testprogramm for RunCode class

    FIRST RELEASE
        2016-07-05  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""
import sys

print(sys.argv[1])
sys.stdout.flush()
print(sys.argv[2], file=sys.stderr)
sys.stderr.flush()

sys.exit(4)
# vim: ft=python ts=4 sta sw=4 et ai



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        run_stdin.py -- Testprogramm for RunCode class

    FIRST RELEASE
        2016-07-05  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""
import sys

VAL = input()  # expact "val1 val2"

SPVAL = VAL.split(" ")

print(SPVAL[0])
sys.stdout.flush()
print(SPVAL[1], file=sys.stderr)
sys.stderr.flush()

sys.exit(3)
# vim: ft=python ts=4 sta sw=4 et ai



"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        __init__.py

    FIRST RELEASE
        2016-06-29  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""
import pkg_resources
try:
    __version__ = pkg_resources.get_distribution(__name__).version  # noqa  # pylint: disable=E1101
except:  # pragma: no cover  # pylint: disable=W0702
    __version__ = 'unknown'  # pragma: no cover

from .returnobject import *  # pylint: disable=W0401,C0413
from .runcode import *  # pylint: disable=W0401,C0413
from .emitter import *  # pylint: disable=W0401,C0413
from .helper import exchange_values  # pylint: disable=C0413

__all__ = (returnobject.__all__ +  # pylint: disable=E0602
           runcode.__all__ +  # pylint: disable=E0602
           helper.__all__ +  # pylint: disable=E0602
           emitter.__all__)  # pylint: disable=E0602
# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        emitter.py -- Emit messages

    FIRST RELEASE
        2016-07-06  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""
from enum import IntEnum, unique
from cuebung_helper import __version__

__author__ = "Sebastian Stigler"
__copyright__ = "Sebastian Stigler"
__license__ = "mit"
__all__ = ['emit', 'set_emit', 'MsgType']


@unique
class MsgType(IntEnum):
    """Message types"""
    message = 0
    passed = 1
    failed = 2
    failed_timedout = 3
    failed_returncode = 4
    failed_output_linecount = 5
    failed_stderr = 6
    failed_stdout = 7
    failed_error = 8


class Emitter():
    """Emitter class with singleton pattern"""
    emit = None

    def __init__(self):
        cls = self.__class__
        if cls.emit is None:  # pragma: no cover
            cls.emit = cls.default_emit

    def __call__(self, msg, *args, msgtype=MsgType.message, **kwargs):
        """Call the emit function"""
        assert isinstance(msg, dict)
        _msg = msg.copy()
        _msg['msgtype'] = msgtype
        cls = self.__class__
        return cls.emit(_msg, **kwargs)

    @staticmethod
    def default_emit(msg, **kwargs):  # pylint: disable=W0613
        """Emit the message dict"""
        print(msg)


emit = Emitter()  # pylint: disable=C0103


def set_emit(func):
    """Set a custom emit function"""
    Emitter.emit = func


def reset_emit():
    """reset the default emit function"""
    Emitter.emit = Emitter.default_emit

# vim: ft=python ts=4 sta sw=4 et ai
# python: 3



"""
    COPYRIGHT (C) 2016 by Sebastian Stigler

    NAME
        helper.py -- helper functions for testing c funktions

    FIRST RELEASE
        2016-06-29  Sebastian Stigler  sebastian.stigler@hs-aalen.de

"""
import abc
from collections import namedtuple
from contextlib import contextmanager
import os
from threading import Thread
from cuebung_helper import __version__

__all__ = ['exchange_values']
__author__ = "Sebastian Stigler"
__copyright__ = "Sebastian Stigler"
__license__ = "mit"

Command = namedtuple('Command', ['ldpreload', 'filename'])
Config = namedtuple('Config', ['use_stdin', 'command', 'args'])


def _islistofstrings(arg):
    """Check if `arg` is a list of strings or an empty list"""
    ret = False
    if isinstance(arg, list):
        ret = all(isinstance(entry, str) for entry in arg)
    return ret


def _cmplistofstrings(list_a, list_b):
    """Compare two list of strings for equaltity"""
    if len(list_a) != len(list_b):
        return False
    if not all(str_a == str_b for str_a, str_b in zip(list_a, list_b)):
        return False
    return True


def _cleanresult(result, encoding='utf-8', errors='replace', strip='\n'):
    """Convert the stdout/stderr from popen in a list of strings"""
    return _lob2los(result.readlines(), encoding, errors, strip)


def _lob2los(listofbytes, encoding='utf-8', errors='replace', strip='\n'):
    """Convert a list of bytestrings in a list of strings"""
    output = []
    for line in listofbytes:
        sline = line.decode(encoding, errors=errors).strip(strip)
        if len(sline) > 0:
            output.append(sline)
    return output


class AutoStorage:
    """Base Descriptor"""
    __counter = 0
    _default = None

    def __init__(self, value=None):
        self.other_value = value
        cls = self.__class__
        prefix = cls.__name__
        index = cls.__counter  # pylint: disable=W0212
        self.storage_name = '_{}#{}'.format(prefix, index)
        cls.__counter += 1

    def __get__(self, instance, owner):
        if instance is None:  # pragma: no cover
            return self  # pragma: no cover
        else:
            return getattr(instance, self.storage_name, self.other_value)

    def __set__(self, instance, value):
        setattr(instance, self.storage_name, value)


class Validated(abc.ABC, AutoStorage):
    """Descriptor class with abstract validator"""

    def __init__(self, value=None):
        value = self.validate(None, value)
        super().__init__(value)
        self.other_value = value or self._default

    def __set__(self, instance, value):
        value = self.validate(instance, value)
        super().__set__(instance, value)

    @abc.abstractmethod
    def validate(self, instance, value):
        """return validated value or raise an ValueError"""


class Boolean(Validated):
    """Boolean Validator"""
    _default = False

    def validate(self, instance, value):
        """return validated value or raise an ValueError"""
        if isinstance(value, bool) or (instance is None and value is None):
            return value
        else:
            raise ValueError('value must be a bool')


class Text(Validated):
    """Text Validator"""

    def validate(self, instance, value):
        """return validated value or raise an ValueError"""
        if isinstance(value, str) or (instance is None and value is None):
            return value
        else:
            raise ValueError('value must be a string')


@contextmanager
def exchange_values(function, args, valuelist):
    """A contextmanager to exchange data via named pipes"""
    fifo_names = {'msg': 'message', 'cmd': 'command'}
    fifo_fds = {'msg': None, 'cmd': None}

    for key in fifo_names:
        if os.path.exists(fifo_names[key]):
            os.unlink(fifo_names[key])  # pragma: no cover
        if not os.path.exists(fifo_names[key]):  # pragma: no cover
            os.mkfifo(fifo_names[key], 0o600)

    def run():
        """thread"""
        fifo_fds['msg'] = os.open(fifo_names['msg'], os.O_RDWR)
        # print('msg open')
        fifo_fds['cmd'] = os.open(fifo_names['cmd'], os.O_RDONLY)
        # print('cmd open')
        try:
            while os.read(fifo_fds['cmd'], 1) == b'N':
                # print('NEXT')
                val = function(*args)
                valuelist.append(val)
                os.write(fifo_fds['msg'], ('%010d' % val).encode())
        except IOError:
            pass

    Thread(target=run).start()

    yield

    hfd = os.open(fifo_names['cmd'], os.O_RDWR)
    os.write(hfd, b'X')
    os.close(hfd)

    for key in fifo_names:
        if fifo_fds[key] is not None:
            os.close(fifo_fds[key])
        if os.path.exists(fifo_names[key]):  # pragma: no cover
            os.unlink(fifo_names[key])


# vim: ft=python ts=4 sta sw=4 et ai
# python: 3
